@@ -58,6 +58,7 @@ struct _IndicatorDatetimePanel
 
 struct _IndicatorDatetimePanelPrivate
 {
+  guint                name_watch_id;
   GtkBuilder *         builder;
   GDBusProxy *         proxy;
   GtkWidget *          auto_radio;
@@ -69,9 +70,8 @@ struct _IndicatorDatetimePanelPrivate
   gboolean             user_edited_time;
   gboolean             changing_time;
   GtkWidget *          loc_dlg;
+  GSettings *          settings;
   CcTimezoneCompletion * completion;
-  GCancellable         * tz_query_cancel;
-  GCancellable         * ntp_query_cancel;
 };
 
 struct _IndicatorDatetimePanelClass
@@ -177,7 +177,8 @@ polkit_perm_ready (GObject *source_object, GAsyncResult *res, gpointer user_data
   GPermission * permission = polkit_permission_new_finish (res, &error);
 
   if (error != NULL) {
-    g_warning ("Could not get permission object: %s", error->message);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Could not get permission object: %s", error->message);
     g_error_free (error);
     return;
   }
@@ -193,7 +194,8 @@ dbus_set_answered (GObject *object, GAsyncResult *res, gpointer command)
   GVariant * answers = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), res, &error);
 
   if (error != NULL) {
-    g_warning("Could not set '%s' for SettingsDaemon: %s", (gchar *)command, error->message);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning("Could not set '%s' using timedated: %s", (gchar *)command, error->message);
     g_error_free(error);
     return;
   }
@@ -206,39 +208,14 @@ toggle_ntp (GtkWidget * radio, GParamSpec * pspec, IndicatorDatetimePanel * self
 {
   gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio));
 
-  g_dbus_proxy_call (self->priv->proxy, "SetUsingNtp", g_variant_new ("(b)", active),
-                     G_DBUS_CALL_FLAGS_NONE, -1, NULL, dbus_set_answered, "using_ntp");
-}
-
-static void
-ntp_query_answered (GObject *object, GAsyncResult *res, IndicatorDatetimePanel * self)
-{
-  GError * error = NULL;
-  GVariant * answers = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), res, &error);
-
-  g_clear_object (&self->priv->ntp_query_cancel);
-
-  if (error != NULL) {
-    g_warning("Could not query DBus proxy for SettingsDaemon: %s", error->message);
-    g_error_free(error);
-    return;
-  }
-
-  gboolean can_use_ntp, is_using_ntp;
-  g_variant_get (answers, "(bb)", &can_use_ntp, &is_using_ntp);
-
-  gtk_widget_set_sensitive (GTK_WIDGET (self->priv->auto_radio), can_use_ntp);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->auto_radio), is_using_ntp);
-
-  g_signal_connect (self->priv->auto_radio, "notify::active", G_CALLBACK (toggle_ntp), self);
-
-  g_variant_unref (answers);
+  g_dbus_proxy_call (self->priv->proxy, "SetNTP", g_variant_new ("(bb)", active, TRUE),
+                     G_DBUS_CALL_FLAGS_NONE, -1, NULL, dbus_set_answered, "NTP");
 }
 
 static void
 sync_entry (IndicatorDatetimePanel * self, const gchar * location)
 {
-  gchar * name = get_current_zone_name (location);
+  gchar * name = get_current_zone_name (location, self->priv->settings);
   gtk_entry_set_text (GTK_ENTRY (self->priv->tz_entry), name);
   g_free (name);
 
@@ -255,7 +232,7 @@ tz_changed (CcTimezoneMap * map, CcTimezoneLocation * location, IndicatorDatetim
   gchar * zone;
   g_object_get (location, "zone", &zone, NULL);
 
-  g_dbus_proxy_call (self->priv->proxy, "SetTimezone", g_variant_new ("(s)", zone),
+  g_dbus_proxy_call (self->priv->proxy, "SetTimezone", g_variant_new ("(sb)", zone, TRUE),
                      G_DBUS_CALL_FLAGS_NONE, -1, NULL, dbus_set_answered, "timezone");
 
   sync_entry (self, zone);
@@ -263,85 +240,81 @@ tz_changed (CcTimezoneMap * map, CcTimezoneLocation * location, IndicatorDatetim
   g_free (zone);
 }
 
-static void
-tz_query_answered (GObject *object, GAsyncResult *res, IndicatorDatetimePanel * self)
-{
-  GError * error = NULL;
-  GVariant * answers = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), res, &error);
-
-  g_clear_object (&self->priv->tz_query_cancel);
-
-  if (error != NULL) {
-    g_warning("Could not query DBus proxy for SettingsDaemon: %s", error->message);
-    g_error_free(error);
-    return;
-  }
-
-  const gchar * timezone;
-  g_variant_get (answers, "(&s)", &timezone);
-
-  cc_timezone_map_set_timezone (self->priv->tzmap, timezone);
-
-  sync_entry (self, timezone);
-  g_signal_connect (self->priv->tzmap, "location-changed", G_CALLBACK (tz_changed), self);
-
-  g_variant_unref (answers);
-}
-
 static void
 proxy_ready (GObject *object, GAsyncResult *res, IndicatorDatetimePanel * self)
 {
   GError * error = NULL;
   IndicatorDatetimePanelPrivate * priv = self->priv;
+  GVariant *value;
 
   self->priv->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 
   if (error != NULL) {
-    g_critical("Could not grab DBus proxy for SettingsDaemon: %s", error->message);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_critical("Could not grab DBus proxy for timedated: %s", error->message);
     g_error_free(error);
     return;
   }
 
   /* And now, do initial proxy configuration */
-  if (priv->ntp_query_cancel == NULL) {
-    priv->ntp_query_cancel = g_cancellable_new();
-    g_dbus_proxy_call (priv->proxy, "GetUsingNtp", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
-                       priv->ntp_query_cancel, (GAsyncReadyCallback)ntp_query_answered, self);
-  }
-  if (priv->tz_query_cancel == NULL); {
-    priv->tz_query_cancel = g_cancellable_new();
-    g_dbus_proxy_call (priv->proxy, "GetTimezone", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
-                       priv->tz_query_cancel, (GAsyncReadyCallback)tz_query_answered, self);
-  }
+  value = g_dbus_proxy_get_cached_property (priv->proxy, "CanNTP");
+  if (value != NULL)
+    {
+      if (g_variant_is_of_type (value, G_VARIANT_TYPE_BOOLEAN))
+        gtk_widget_set_sensitive (priv->auto_radio, g_variant_get_boolean (value));
+      g_variant_unref (value);
+    }
+
+  value = g_dbus_proxy_get_cached_property (priv->proxy, "NTP");
+  if (value != NULL)
+    {
+      if (g_variant_is_of_type (value, G_VARIANT_TYPE_BOOLEAN))
+        {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->auto_radio), g_variant_get_boolean (value));
+          g_signal_connect (priv->auto_radio, "notify::active", G_CALLBACK (toggle_ntp), self);
+        }
+      g_variant_unref (value);
+    }
+
+  value = g_dbus_proxy_get_cached_property (priv->proxy, "Timezone");
+  if (value != NULL)
+    {
+      if (g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
+        {
+          const gchar *timezone = g_variant_get_string (value, NULL);
+
+          cc_timezone_map_set_timezone (priv->tzmap, timezone);
+          sync_entry (self, timezone);
+          g_signal_connect (priv->tzmap, "location-changed", G_CALLBACK (tz_changed), self);
+        }
+      g_variant_unref (value);
+    }
+}
+
+#define WIG(name) GTK_WIDGET (gtk_builder_get_object(self->priv->builder, name))
+
+static void
+set_show_clock_check_sensitive (IndicatorDatetimePanel  * self,
+                                gboolean                  sensitive)
+{
+  gtk_widget_set_sensitive (WIG("showClockCheck"), sensitive);
 }
 
 static void
-service_name_owner_changed (GDBusProxy * proxy, GParamSpec *pspec, gpointer user_data)
+on_bus_name_appeared (GDBusConnection * connection   G_GNUC_UNUSED,
+                      const char      * name         G_GNUC_UNUSED,
+                      const char      * name_owner,
+                      gpointer          self)
 {
-  GtkWidget * widget = GTK_WIDGET (user_data);
-  gchar * owner = g_dbus_proxy_get_name_owner (proxy);
-
-  gtk_widget_set_sensitive (widget, (owner != NULL));
-
-  g_free (owner);
+  set_show_clock_check_sensitive (self, name_owner && *name_owner);
 }
 
 static void
-service_proxy_ready (GObject *object, GAsyncResult *res, gpointer user_data)
+on_bus_name_vanished (GDBusConnection * connection   G_GNUC_UNUSED,
+                      const char      * name         G_GNUC_UNUSED,
+                      gpointer          self)
 {
-  GError * error = NULL;
-
-  GDBusProxy * proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-  if (error != NULL) {
-    g_critical("Could not grab DBus proxy for indicator-datetime-service: %s", error->message);
-    g_error_free(error);
-    return;
-  }
-
-  /* And now, do initial proxy configuration */
-  g_signal_connect (proxy, "notify::g-name-owner", G_CALLBACK (service_name_owner_changed), user_data);
-  service_name_owner_changed (proxy, NULL, user_data);
+  set_show_clock_check_sensitive (self, FALSE);
 }
 
 static gboolean
@@ -359,7 +332,8 @@ save_time (IndicatorDatetimePanel * self)
 {
   if (self->priv->user_edited_time) {
     gdouble current_value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (self->priv->date_spin));
-    g_dbus_proxy_call (self->priv->proxy, "SetTime", g_variant_new ("(x)", (guint64)current_value),
+    g_dbus_proxy_call (self->priv->proxy, "SetTime",
+                       g_variant_new ("(xbb)", (gint64) (current_value * G_TIME_SPAN_SECOND), FALSE, TRUE),
                        G_DBUS_CALL_FLAGS_NONE, -1, NULL, dbus_set_answered, "time");
   }
   self->priv->user_edited_time = FALSE;
@@ -498,7 +472,10 @@ format_time_text (GtkWidget * spinner, gpointer user_data)
     }
   }
   else {
-    format = "%x";
+    // This is intentionally not "%x".  See https://launchpad.net/bugs/1149696
+    // If you are willing to do the hard work of writing a locale-sensitive
+    // date parser, there is an open bug: https://launchpad.net/bugs/729056
+    format = "%Y-%m-%d";
   }
 
   GDateTime * datetime = g_date_time_new_from_unix_local (gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinner)));
@@ -623,11 +600,9 @@ timezone_selected (GtkEntryCompletion * widget, GtkTreeModel * model,
     zone = cc_timezone_map_get_timezone_at_coords (self->priv->tzmap, lon, lat);
   }
 
-  GSettings * conf = g_settings_new (SETTINGS_INTERFACE);
   gchar * tz_name = g_strdup_printf ("%s %s", zone, name);
-  g_settings_set_string (conf, SETTINGS_TIMEZONE_NAME_S, tz_name);
+  g_settings_set_string (self->priv->settings, SETTINGS_TIMEZONE_NAME_S, tz_name);
   g_free (tz_name);
-  g_object_unref (conf);
 
   cc_timezone_map_set_timezone (self->priv->tzmap, zone);
 
@@ -647,7 +622,7 @@ entry_focus_out (GtkEntry * entry, GdkEventFocus * event, IndicatorDatetimePanel
   gchar * zone;
   g_object_get (location, "zone", &zone, NULL);
 
-  gchar * name = get_current_zone_name (zone);
+  gchar * name = get_current_zone_name (zone, self->priv->settings);
   gboolean correct = (g_strcmp0 (gtk_entry_get_text (entry), name) == 0);
   g_free (name);
   g_free (zone);
@@ -663,14 +638,18 @@ entry_focus_out (GtkEntry * entry, GdkEventFocus * event, IndicatorDatetimePanel
 static void
 indicator_datetime_panel_init (IndicatorDatetimePanel * self)
 {
+  GError * error;
+  GSettings * conf;
+
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                             INDICATOR_DATETIME_TYPE_PANEL,
                                             IndicatorDatetimePanelPrivate);
 
-  GError * error = NULL;
+  self->priv->settings = conf = g_settings_new (SETTINGS_INTERFACE);
 
   self->priv->builder = gtk_builder_new ();
   gtk_builder_set_translation_domain (self->priv->builder, GETTEXT_PACKAGE);
+  error = NULL;
   gtk_builder_add_from_file (self->priv->builder, DATETIME_DIALOG_UI_FILE, &error);
   if (error != NULL) {
     /* We have to abort, we can't continue without the ui file */
@@ -679,9 +658,6 @@ indicator_datetime_panel_init (IndicatorDatetimePanel * self)
     return;
   }
 
-  GSettings * conf = g_settings_new (SETTINGS_INTERFACE);
-
-#define WIG(name) GTK_WIDGET (gtk_builder_get_object (self->priv->builder, name))
 
   /* Add policykit button */
   GtkWidget * polkit_button = gtk_lock_button_new (NULL);
@@ -693,7 +669,7 @@ indicator_datetime_panel_init (IndicatorDatetimePanel * self)
   gtk_container_add (GTK_CONTAINER (alignment), polkit_button);
   gtk_box_pack_start (GTK_BOX (WIG ("timeDateBox")), alignment, FALSE, TRUE, 0);
 
-  const gchar * polkit_name = "org.gnome.settingsdaemon.datetimemechanism.configure";
+  const gchar * polkit_name = "org.gnome.controlcenter.datetime.configure";
   polkit_permission_new (polkit_name, NULL, NULL, polkit_perm_ready, polkit_button);
 
   /* Add map */
@@ -760,9 +736,9 @@ indicator_datetime_panel_init (IndicatorDatetimePanel * self)
 
   /* Grab proxy for settings daemon */
   g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM, G_DBUS_PROXY_FLAGS_NONE, NULL,
-                            "org.gnome.SettingsDaemon.DateTimeMechanism",
-                            "/",                            
-                            "org.gnome.SettingsDaemon.DateTimeMechanism",
+                            "org.freedesktop.timedate1",
+                            "/org/freedesktop/timedate1",
+                            "org.freedesktop.timedate1",
                             NULL, (GAsyncReadyCallback)proxy_ready, self);
 
   /* Grab proxy for datetime service, to see if it's running.  It would
@@ -770,15 +746,16 @@ indicator_datetime_panel_init (IndicatorDatetimePanel * self)
      but that doesn't yet claim a name on the bus.  Presumably the service
      would have been started by any such indicator, so this will at least tell
      us if there *was* a datetime module run this session. */
-  g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION, G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START, NULL,
-                            SERVICE_NAME, SERVICE_OBJ, SERVICE_IFACE,
-                            NULL, (GAsyncReadyCallback)service_proxy_ready,
-                            WIG ("showClockCheck"));
+  self->priv->name_watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                                BUS_NAME,
+                                                G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                                on_bus_name_appeared,
+                                                on_bus_name_vanished,
+                                                self,
+                                                NULL);
 
 #undef WIG
 
-  g_object_unref (conf);
-
   gtk_widget_show_all (panel);
   gtk_container_add (GTK_CONTAINER (self), panel);
 }
@@ -791,22 +768,18 @@ indicator_datetime_panel_dispose (GObject * object)
 
   g_clear_object (&priv->builder);
   g_clear_object (&priv->proxy);
-
-  if (priv->tz_query_cancel != NULL) {
-    g_cancellable_cancel (priv->tz_query_cancel);
-    g_clear_object (&priv->tz_query_cancel);
-  }
-
-  if (priv->ntp_query_cancel != NULL) {
-    g_cancellable_cancel (priv->ntp_query_cancel);
-    g_clear_object (&priv->ntp_query_cancel);
-  }
+  g_clear_object (&priv->settings);
 
   if (priv->loc_dlg) {
     gtk_widget_destroy (priv->loc_dlg);
     priv->loc_dlg = NULL;
   }
 
+  if (priv->name_watch_id != 0) {
+    g_bus_unwatch_name (priv->name_watch_id);
+    priv->name_watch_id = 0;
+  }
+
   if (priv->save_time_id) {
     g_source_remove (priv->save_time_id);
     priv->save_time_id = 0;
@@ -840,13 +813,22 @@ indicator_datetime_panel_class_finalize (IndicatorDatetimePanelClass *klass)
 {
 }
 
+static const char *
+indicator_datetime_panel_get_help_uri (CcPanel *panel)
+{
+  return "help:ubuntu-help/clock";
+}
+
 static void
 indicator_datetime_panel_class_init (IndicatorDatetimePanelClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (IndicatorDatetimePanelPrivate));
 
+  panel_class->get_help_uri = indicator_datetime_panel_get_help_uri;
+
   gobject_class->dispose = indicator_datetime_panel_dispose;
 }
 
